function pa=fevo_parents(f)
% function pa=fevo_parents(f)
%
% Sample (genderless) parent-pairs of children for evolutionary modelling
%
% (Relatively) fast handily programmed algorithm to draw parent-pairs for children from same number of adults
% Basically corresponds to a specific, manually programmed (relatively) fast algorithm for WEIGHTED picking WITHOUT REPLACEMENT
%
% Assigns each child two non-identical parents (pa) according to fitness (sampling weight) (f), gender-unspecific
%
% f = fitness array, dimension n x m, with n = number independent groups, m = number individuals per group
%
% pa = parents assigned to children, dimension m x 2 x n, thus for each independent group n : an array of dimension m x 2 attributing 2 parents to each of the m children
%
% On alternatives: Matlab's randsample does not allow to combine weighting with no-replacement - and I'm not sure datasample would or whether it would do so efficiently as it is more general; it is not available to me
%
% Speed : I consider this algorithm as fast as with randsample I'm not faster even if I ignore the no-replacement condition:
%  randsample, ignoring no-replacement requirement:
%     f=rand(400,20);tic;for i=1:400,pa(:,1,i)=randsample(20,20,true,f(i,:));pa(:,2,i)=randsample(20,20,true,f(i,:));end;toc
%  => 0.07 seconds
%
%  fevo_parents, accounting for no-replacement requirement:
%     f=rand(400,20);tic;pa=fevo_parents(f);toc;
%  => 0.07 seconds as well
%
% Florian Habermacher, 1 Jan 2015, florian.habermacher@gmail.com.

%Parameters
[n,m]=size(f); %n : number groups, m : number individuals per group
%Auxiliaries
ones1m=ones(1,m);
onesm1=ones(m,1);
ones1mm1=ones(1,m-1);
triump1=triu(ones(m),+1);
triumm1p1=triu(ones(m-1),+1);
shixtoim=[0:m:m*(m-1)]'; %"shifting of indexes to translate relative indexes row-by-row into absolute indexes of vectorized m*m matrix
pa=NaN(m,2,n);
%Fundamental randoms parent selection
parndi=rand(m,2,n);
%Calculate
for i=1:n
  %Weights (unscaled) = fitness
  w=f(i,:);
  %Weights ww(all parents, same for each child)
  ww=onesm1*w;
  parnd=parndi(:,:,i);
  %First parent easily - scale fundamental randoms and look which position this corresponds
  parnd1=sum(w)*parnd(:,1);
  wwcum=ww*triump1;
  p1=sum((parnd1*ones1m)>wwcum,2);
  pa(:,1,i)=p1;
  %Second parent slightly more complex but still simple
  % Main idea to preserve matrix speeds: use shixtoim index shift to express individual elements to delete in the individual rows as positions within the vectorized matrix, erase these elements with the "=[]" assignment, and re-matricise the vector using vec2mat!
  wwtemp=ww';
  wwtemp(p1+shixtoim)=[];
  ww2=vec2mat(wwtemp,m-1);
  parnd2=sum(ww2,2).*parnd(:,2);
  ww2cum=ww2*triumm1p1;
  p2_pre=sum((parnd2*ones1mm1)>ww2cum,2);
  p2=p2_pre+(p2_pre>=p1);
  pa(:,2,i)=p2;
end;